<?php

/*
	@link              http://www.pixelsandthings.co.uk
	@since             1.0.0
	@package           Fetch_Some_Tweets

	@fetch-some-tweets-plugin
	Plugin Name:       Fetch Some Tweets
	Plugin URI:        http://www.pixelsandthings.co.uk
	Description:       JSON Driven PHP Twitter Feed via Twitter API
	Version:           1.5
	Author:            Pixels & Things
	Author URI:        https://www.pixelsandthings.co.uk
	License:           GPL-2.0+
	License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
	Text Domain:       fetch-some-tweets
*/


/* -----------------------------------------------------------------------------
Security Check
----------------------------------------------------------------------------- */

if( ! defined( 'ABSPATH' ) ){
	die();
}


/* -----------------------------------------------------------------------------
Plugin Version - https://semver.org
----------------------------------------------------------------------------- */

define( 'PLUGIN_NAME_VERSION', '1.3' );


/* -----------------------------------------------------------------------------
Include Twitter OAuth Library - https://twitteroauth.com
----------------------------------------------------------------------------- */

if ( is_readable( __DIR__ . '/vendor/autoload.php' ) ) {
    require __DIR__ . '/vendor/autoload.php';
}

use Abraham\TwitterOAuth\TwitterOAuth;


/* -----------------------------------------------------------------------------
Initialize
----------------------------------------------------------------------------- */

class FetchSomeTweetsPlugin{

	//Set Plugin Basename
	function __construct() {
		$this->plugin = plugin_basename( __FILE__ );
	}

	//Activation
	function activate() {
		//Preperation for V1.1
		flush_rewrite_rules();
	}

	//Deactivation
	function deactivate() {
		//Preperation for V1.1
		flush_rewrite_rules();
	}

	//Uninstallation
	function uninstall() {
		//Preperation for V1.1
		flush_rewrite_rules();
	}

	//Register Actions & Filters
	function register(){
		add_action( 'admin_enqueue_scripts', array( $this, 'fst_enqueue_scripts' ) );
		add_action( 'admin_menu', array( $this, 'fst_register_options_page' ) );
		add_action( 'admin_init', array( $this, 'fst_register_settings' ) );
		add_filter( "plugin_action_links_$this->plugin", array( $this, 'fst_settings_link') );
		add_shortcode('FetchSomeTweets', array( $this, 'FetchSomeTweets' ) );
	}

	//Enqueue Admin CSS
	function fst_enqueue_scripts() {
		wp_enqueue_style(  'fetchsometweets_css', plugins_url( '/assets/css/fst_style.css', __FILE__ ) );
	}

	//Add 'Settings/Options' link to Plugins Page
	function fst_settings_link ( $links ) {
		$mylinks = array(
			'<a href="' . admin_url( "options-general.php?page=fetch_some_tweets" ) . '">Settings</a>',
		);
		return array_merge( $links, $mylinks );
	}


	/* -----------------------------------------------------------------------------
	Settings Page
	----------------------------------------------------------------------------- */

	//Add 'Settings/Options' Page Link within 'Settings'
	function fst_register_options_page() {
		add_options_page( 'Fetch Some Tweets Settings', 'Fetch Some Tweets', 'manage_options', 'fetch_some_tweets', array( $this, 'fst_options_page' ) );
	}

	//Register 'Settings/Options' Page
	function fst_register_settings() {

		register_setting( 'optionsPage', 'fst_settings' );

		add_settings_section(
			'fst_optionsPage_section',
			__( 'Your Twitter Developer Authentication Details', 'fetch-some-tweets' ),
			array( $this, 'fst_settings_section_callback' ),
			'optionsPage'
		);

		add_settings_field(
			'fst_text_field_0',
			__( 'Consumer Key', 'fetch-some-tweets' ),
			array( $this, 'fst_text_field_0_render' ),
			'optionsPage',
			'fst_optionsPage_section'
		);

		add_settings_field(
			'fst_text_field_1',
			__( 'Consumer Secret', 'fetch-some-tweets' ),
			array( $this, 'fst_text_field_1_render' ),
			'optionsPage',
			'fst_optionsPage_section'
		);

		add_settings_field(
			'fst_text_field_2',
			__( 'Access Token', 'fetch-some-tweets' ),
			array( $this,  'fst_text_field_2_render' ),
			'optionsPage',
			'fst_optionsPage_section'
		);

		add_settings_field(
			'fst_text_field_3',
			__( 'Consumer Token Secret', 'fetch-some-tweets' ),
			array( $this, 'fst_text_field_3_render' ),
			'optionsPage',
			'fst_optionsPage_section'
		);

		add_settings_field(
			'fst_text_field_4',
			__( 'Twitter Handle @', 'fetch-some-tweets' ),
			array( $this, 'fst_text_field_4_render' ),
			'optionsPage',
			'fst_optionsPage_section'
		);

		add_settings_field(
			'fst_text_field_5',
			__( 'Number of Tweets', 'fetch-some-tweets' ),
			array( $this, 'fst_text_field_5_render' ),
			'optionsPage',
			'fst_optionsPage_section'
		);

		add_settings_field(
			'fst_text_field_6',
			__( 'Include Retweets', 'fetch-some-tweets' ),
			array( $this, 'fst_text_field_6_render' ),
			'optionsPage',
			'fst_optionsPage_section'
		);

		add_settings_field(
			'fst_text_field_7',
			__( 'Exclude Replies', 'fetch-some-tweets' ),
			array( $this, 'fst_text_field_7_render' ),
			'optionsPage',
			'fst_optionsPage_section'
		);
	}

	//Register 'Settings/Options' Page Form Fields
	function fst_text_field_0_render() {
		$options = get_option( 'fst_settings' );
		?>
		<input type='text' name='fst_settings[fst_text_field_0]' value='<?php if( isset( $options['fst_text_field_0'] ) ){ echo $options['fst_text_field_0']; } ?>'>
		<?php
	}

	function fst_text_field_1_render() {
		$options = get_option( 'fst_settings' );
		?>
		<input type='text' name='fst_settings[fst_text_field_1]' value='<?php if( isset( $options['fst_text_field_1'] ) ){ echo $options['fst_text_field_1']; } ?>'>
		<?php
	}

	function fst_text_field_2_render() {
		$options = get_option( 'fst_settings' );
		?>
		<input type='text' name='fst_settings[fst_text_field_2]' value='<?php if( isset( $options['fst_text_field_2'] ) ){ echo $options['fst_text_field_2']; } ?>'>
		<?php
	}

	function fst_text_field_3_render() {
		$options = get_option( 'fst_settings' );
		?>
		<input type='text' name='fst_settings[fst_text_field_3]' value='<?php if( isset( $options['fst_text_field_3'] ) ){ echo $options['fst_text_field_3']; } ?>'>
		<?php
	}

	function fst_text_field_4_render() {
		$options = get_option( 'fst_settings' );
		?>
		<input type='text' name='fst_settings[fst_text_field_4]' value='<?php if( isset( $options['fst_text_field_4'] ) ){ echo $options['fst_text_field_4']; } ?>'>
		<?php
	}

	function fst_text_field_5_render() {
		$options = get_option( 'fst_settings' );
		?>
		<input type='text' name='fst_settings[fst_text_field_5]' value='<?php if( isset( $options['fst_text_field_5'] ) ){ echo $options['fst_text_field_5']; } ?>'>
		<?php
	}

	function fst_text_field_6_render() {
		$options = get_option( 'fst_settings' );
		if( !isset( $options['fst_text_field_6'] ) ) $options['fst_text_field_6'] = 0;
	    $html = '<input type="checkbox" name="fst_settings[fst_text_field_6]" value="1"' . checked( 1, $options['fst_text_field_6'], false ) . '/>';
	    echo $html;
	}

	function fst_text_field_7_render() {
		$options = get_option( 'fst_settings' );
		if( !isset( $options['fst_text_field_7'] ) ) $options['fst_text_field_7'] = 0;
	    $html = '<input type="checkbox" name="fst_settings[fst_text_field_7]" value="1"' . checked( 1, $options['fst_text_field_7'], false ) . '/>';
	    echo $html;
	}

	function fst_settings_section_callback() {
		echo __(
		'<p>In order to use this plugin you must have created a Twitter Developers Account via the <a href="https://developer.twitter.com/content/developer-twitter/en.html" target="_blank">Twitter Developer Platform</a> and also created a <a href="https://developer.twitter.com/en/apps" target="_blank">Twitter App</a>.</p>
		<p>Instructions on how to create a Twitter App can be found <a href="https://developer.twitter.com/en/docs/basics/developer-portal/guides/apps" target="_blank">here</a>.</p>
		<p>Instructions on how to generate your Access Tokens can be found <a href="https://developer.twitter.com/en/docs/basics/authentication/guides/access-tokens.html" target="_blank">here</a>.</p>
		<p>To show the Tweets within your templates simply fill out the fields below and use the following shortcode <strong>[FetchSomeTweets]</strong>.</p>', 'fetch-some-tweets'
		);
	}

	//Register 'Settings/Options' Page Content
	function fst_options_page() {
		?>
		<div class="wrap">
			<h2>Fetch Some Tweets</h2>
			<form action='options.php' method='post'>
				<?php
					settings_fields( 'optionsPage' );
					do_settings_sections( 'optionsPage' );
					submit_button();
				?>
			</form>
		</div>
		<?php
	}


	/* -----------------------------------------------------------------------------
	Twitter Fetcher
	----------------------------------------------------------------------------- */

	function FetchSomeTweets(){

		// Key & Tokens
		$options = get_option( 'fst_settings' );
		
		if(isset($options['fst_text_field_0'])):
			$consumer_key = $options['fst_text_field_0'];
		endif;

		if(isset($options['fst_text_field_1'])):
			$consumer_secret = $options['fst_text_field_1'];
		endif;
		
		if(isset($options['fst_text_field_2'])):
			$access_token = $options['fst_text_field_2'];
		endif;

		if(isset($options['fst_text_field_3'])):
			$access_token_secret = $options['fst_text_field_3'];
		endif;

		// User Details
		if(isset($options['fst_text_field_4'])):
			$screen_name = $options['fst_text_field_4'];
		endif;

		// Tweet Count
		if(isset($options['fst_text_field_5'])):
			$tweet_count = $options['fst_text_field_5'];
		endif;

		// Timeline Options

		// Include Re-Tweets
		if(isset($options['fst_text_field_6']) && $options['fst_text_field_6'] == '1'){
			$include_retweets = 'true';
		} else {
			$include_retweets = 'false';
		}
		
		// Exclude Replies
		if(isset($options['fst_text_field_7']) && $options['fst_text_field_7'] == '1'){
			$exclude_replies = 'true';
		} else {
			$exclude_replies = 'false';
		}
		
		// Connect to Twitter API - https://github.com/abraham/twitteroauth
		$connection = new TwitterOAuth(
			$consumer_key,
			$consumer_secret,
			$access_token,
			$access_token_secret
		);

		// Verify Connection, Keys & Tokens
		$content = $connection->get('account/verify_credentials');

		// Verify Connection to Twitter
		if(! $content ){

			$no_fst_twitter_user_timeline = '<section class="no-tweets no-connection">';
			$no_fst_twitter_user_timeline .= '<p>Cannot connect to Twitter, check your internet!</p>';
			$no_fst_twitter_user_timeline .= '</section>';
			return $no_fst_twitter_user_timeline;
			die();

		// Check for Errors - $content
		} elseif( isset( $content->errors ) ){

			// No Tweets
			$no_fst_twitter_user_timeline = '<section class="no-tweets">';
			$no_fst_twitter_user_timeline .= '<p>Check your Key & Tokens, your crendentials could not be authenticated! :(</p>';
			$no_fst_twitter_user_timeline .= '</section>';
			return $no_fst_twitter_user_timeline;
			die();

		} else {

			// Fetch More Tweets than needed at first if excluding replies
			$total_tweets = ($exclude_replies) ? max(50, $tweet_count * 2) : $tweet_count;

			// Get Tweets - User Timeline - https://developer.twitter.com/en/docs/tweets/timelines/api-reference/get-statuses-user_timeline.html
			$tweets = $connection->get('statuses/user_timeline',
				[
					'screen_name' => $screen_name,
					'count' => $total_tweets,
					'trim_user' => false,
					'include_rts' => $include_retweets,
					'exclude_replies' => $exclude_replies,
					'tweet_mode' => 'extended',
				]
			);

			// Check for Errors - $tweets
			if( isset( $tweets->errors ) ){

				// No Tweets
				$no_fst_twitter_user_timeline = '<section class="no-tweets">';
				$no_fst_twitter_user_timeline .= '<p>No Tweets found! :(</p>';
				$no_fst_twitter_user_timeline .= '</section>';
				return $no_fst_twitter_user_timeline;
			
			// Check if Wordpress Cache Transient 'fst_twitter_user_timeline' exists
			} elseif ( false === ( $fst_twitter_user_timeline = get_transient( 'fst_twitter_user_timeline' ) ) ) {

				// Minimum number of Tweets from 'Settings'
				$tweet_limit = min( $tweet_count, count( $tweets ) );

				// DEBUG
				// echo 'Include Re-Tweets: '.$include_retweets.'<br>';
				// echo 'Exclude Replies: '.$exclude_replies.'<br>';
				// echo 'Tweet Count: '.$tweet_count.'<br>';
				// echo 'Total Tweets: '.$total_tweets.'<br>';
				// echo 'Tweet Limit: '.$tweet_limit.'<br>';

				// Tweets
				$fst_twitter_user_timeline .= '<div class="tweet-list">';

				// Final number of Tweets from 'Settings' calculated if excluding replies
				for($i = 0; $i < $tweet_limit; $i++) : $tweet = $tweets[$i];

					// Tweet Date
					$tweet_date = $tweet->created_at;

					//Pass Date & Time
					$passed_tweet_date = date( "l M j", strtotime( $tweet_date ) );

					// Tweet ID
					$tweet_id = $tweet->id;

					// Tweet Content
					if( isset( $tweet->full_text ) ){

						// Standard Tweet Text
						$tweet_text = $tweet->full_text;

					} elseif( isset( $tweet->extended_tweet->full_text ) ) {

						// Extended Tweet Text
						$tweet_text = $tweet->extended_tweet->full_text;

					}

					//Links, Users & Hashtags
					if( isset( $tweet_text ) ){

						//Find Links, Users & Hashtags
						$patterns = array( '@(https?://([-\w\.]+)+(/([\w/_\.]*(\?\S+)?(#\S+)?)?)?)@', '/@(\w+)/', '/\s+#(\w+)/' );
						//Replace Links, Users & Hashtags
						$replacements = array( '<a class="url" href="$1" target="_blank">$1</a>', '<a class="user" href="http://twitter.com/$1" target="_blank">@$1</a>', '<a class="hashtag" href="//twitter.com/hashtag/$1" target="_blank">#$1</a>' );
						//Passed Tweet
						$tweet_text_passed = preg_replace( $patterns, $replacements, $tweet_text );
					}

					// Fetched Twitter Feed User Data - This Data is currently being displayed
					$fst_twitter_user_id = $tweet->user->id;
					$fst_twitter_user_name = $tweet->user->name;
					$fst_twitter_user_screen_name = $tweet->user->screen_name;
					$fst_twitter_user_location = $tweet->user->location;
					$fst_twitter_user_description = $tweet->user->description;
					$fst_twitter_user_verified = $tweet->user->verified;
					$fst_twitter_user_followers = $tweet->user->followers_count;
					$fst_twitter_user_timeline_profile_image = $tweet->user->profile_image_url_https;

					// Retweet & Favorite Count
					$fst_twitter_user_retweet_count = $tweet->retweet_count;
					$fst_twitter_user_favorite_count = $tweet->favorite_count;

					$fst_twitter_user_timeline .= '<article class="tweet" id="' .$tweet_id. '">';

						$fst_twitter_user_timeline .= '<div class="tweet-content">';

							$fst_twitter_user_timeline .= '<div class="tweet-header">';

								// Tweet Type & Status - Tweet, Retweet, Quoted Retweet or Reply
								if( isset($tweet->quoted_status) ){
									$fst_twitter_user_timeline .= '<span class="tweet-type">Quoted Retweet</span> <span class="seperator"> - </span>';
								} elseif( isset($tweet->in_reply_to_user_id) ){
									$fst_twitter_user_timeline .= '<span class="tweet-type">Reply</span> <span class="seperator"> - </span>';
								} else {
									$fst_twitter_user_timeline .= '<span class="tweet-type">Tweet</span> <span class="seperator"> - </span>';
								}

								$fst_twitter_user_timeline .= '<span class="tweet-date">';
								$fst_twitter_user_timeline .= $passed_tweet_date;
								$fst_twitter_user_timeline .= '</span>';

							$fst_twitter_user_timeline .= '</div>';

							$fst_twitter_user_timeline .= '<p class="tweet-text">';
							$fst_twitter_user_timeline .= $tweet_text_passed;
							$fst_twitter_user_timeline .= '</p>';

							// Tweet URLS
							if( isset( $tweet->entities->urls->url ) ){

								// Tiny URL
								$tiny_url = $tweet->entities->urls->url;

								$fst_twitter_user_timeline .= '<a class="tweet-tiny-url" href="' .esc_url( $tiny_url ). '" target="_blank">View Tweet</a>';

							} elseif( isset( $tweet->entities->urls->expanded_url ) ) {

								// Expanded URL
								$expanded_url = $tweet->entities->urls->expanded_url;

								$fst_twitter_user_timeline .= '<a class="tweet-expanded-url" href="' .esc_url( $expanded_url ). '" target="_blank">View Tweet</a>';
							}

							// Interaction Buttons
							$fst_twitter_user_timeline .= '<div class="tweet-interaction">';

							// Media - Images
							if( isset( $tweet->entities->media ) ) {

								// Single Image
								$single_image = $tweet->entities->media;

								$fst_twitter_user_timeline .= '<a class="tweet-button-view-image" href="' .esc_url( $single_image[0]->expanded_url ). '" target="_blank">View Image</a>';
							}

							// Reply
							$fst_twitter_user_timeline .= '<a class="tweet-button-reply" href="https://twitter.com/intent/tweet/?in_reply_to=' .$tweet_id. '" target="_blank">Reply</a>';

							// Retweet
				  			$fst_twitter_user_timeline .= '<a class="tweet-button-retweet" href="https://twitter.com/intent/retweet?tweet_id=' .$tweet_id. '" target="_blank">Retweet</a>';

				  			if( isset( $fst_twitter_user_retweet_count ) && ( $fst_twitter_user_retweet_count > 0 ) ) {
								$fst_twitter_user_timeline .= '<span class="retweet-count">' .$fst_twitter_user_retweet_count. '</span>';
							}

				  			// Favourite
				  			$fst_twitter_user_timeline .= '<a class="tweet-button-favourite" href="https://twitter.com/intent/favorite?tweet_id=' .$tweet_id. '" target="_blank">Favourite</a>';

				  			if( isset( $fst_twitter_user_favorite_count ) && ( $fst_twitter_user_favorite_count > 0 ) ) {
								$fst_twitter_user_timeline .= '<span class="favorite-count">' .$fst_twitter_user_favorite_count. '</span>';
							}

				  			// Send Direct Message - https://developer.twitter.com/en/docs/direct-messages/buttons/overview
				  			$fst_twitter_user_timeline .= '<a class="tweet-button-message" href="https://twitter.com/messages/compose?recipient_id=' .$fst_twitter_user_id. '" target="_blank">Message</a>';

				  			$fst_twitter_user_timeline .= '</div>';

			  			$fst_twitter_user_timeline .= '</div>';

					$fst_twitter_user_timeline .= '</article>';

				endfor;

				$fst_twitter_user_timeline .= '</div>';

				// Feed Owner Data - $screen_name
				$fst_twitter_user_timeline .= '<div class="user-data">';

					$fst_twitter_user_timeline .= '<a class="user-image" href="http://twitter.com/' .$fst_twitter_user_screen_name. '" target="_blank"><img src="' .$fst_twitter_user_timeline_profile_image. '" /></a>';
					$fst_twitter_user_timeline .= '<div class="user-name" id="' .$fst_twitter_user_id. '">' .$fst_twitter_user_name. '</div>';

					if( $fst_twitter_user_verified ){
						$fst_twitter_user_timeline .= '<span class="user-verified">Verified</span>';
					}

					$fst_twitter_user_timeline .= '<div class="user-screen-name"><a href="http://twitter.com/' .$fst_twitter_user_screen_name. '" target="_blank">@' .$fst_twitter_user_screen_name. '</a></div>';
					$fst_twitter_user_timeline .= '<div class="user-location">Location: ' .$fst_twitter_user_location. '</div>';
					$fst_twitter_user_timeline .= '<div class="user-description">Description: ' .$fst_twitter_user_description. '</div>';
					$fst_twitter_user_timeline .= '<div class="user-followers">Followers: ' .$fst_twitter_user_followers. '</div>';

				$fst_twitter_user_timeline .= '</div>';

				// Set Wordpress Cache Transient 'fst_twitter_user_timeline' - define('WP_CACHE', true); in wp-config.php
				set_transient( 'fst_twitter_user_timeline', $fst_twitter_user_timeline, 12 * HOUR_IN_SECONDS );
				
				// DEBUG - Transient Cache Check
				// echo 'Uncached';

				// Return Feed - Uncached
				return $fst_twitter_user_timeline;

				
			
			} else {

				// DEBUG - Transient Cache Check
				// echo 'Cached';

				// Return Feed - Cached
				return $fst_twitter_user_timeline;

			}
		}
	}
}

if ( class_exists( 'FetchSomeTweetsPlugin' ) ){
	$FetchSomeTweetsPlugin = new FetchSomeTweetsPlugin();
	$FetchSomeTweetsPlugin->register();
}

register_deactivation_hook( __FILE__, array( $FetchSomeTweetsPlugin, 'deactivate' ) );
register_activation_hook( __FILE__, array( $FetchSomeTweetsPlugin, 'activate' ) );

?>
